//Thanks hiroi01!

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdbool.h>

#include <pspkernel.h>
#include <psppower.h>
#include <pspctrl.h>
#include <pspsysmem_kernel.h>
#include <pspimpose_driver.h>

#include "libinip.h"

/*----------------------------------------------------------------------------*/

// モジュールの定義
PSP_MODULE_INFO( "slide_enhance", PSP_MODULE_KERNEL, 1, 0 );


/*----------------------------------------------------------------------------*/

#define 	PSP_IMPOSE_SLIDE_ACTION   0x8000000D
#define 	PSP_CTRL_SLIDE            0x20000000

#define INI_PATH "/slide_enhance.ini"

// キー用変数 iniの項目分必要
ILP_Key key[5];

typedef struct CFG{
	bool swap;
	u32 swap_button;
	u32 shutdown_button;
} CFG;

CFG config; 
int slide_action;

/*----------------------------------------------------------------------------*/

int module_start( SceSize arglen, void *argp );
int module_stop( void );
int main_thread( SceSize arglen, void *argp );

/*----------------------------------------------------------------------------*/

void roadIni(const char *path)
{
	char iniPath[256];
	char *temp;
	bool swap[3];
	SceIoStat stat;

	strcpy(iniPath, path);
	temp = strrchr(iniPath, '/');
	if( temp != NULL ) *temp = '\0';
	strcat(iniPath, INI_PATH);
	
	if(sceIoGetstat(iniPath, &stat) == 0){
		config.swap_button = 0;
		config.shutdown_button = 0;
	} else {
		config.swap_button = PSP_CTRL_LTRIGGER;
		config.shutdown_button = PSP_CTRL_RTRIGGER;
	}

	// iniファイルの読み込みの下準備
	ILPInitKey(key);

	ILPRegisterButton(key,	"SwapButton",		&config.swap_button,		config.swap_button,		NULL);
	ILPRegisterButton(key,	"ShutdownButton",	&config.shutdown_button,	config.shutdown_button,	NULL);
	ILPRegisterBool(key,	"Swap_vsh",		&swap[0],	false);
	ILPRegisterBool(key,	"Swap_game",	&swap[1],	false);
	ILPRegisterBool(key,	"Swap_pops",	&swap[2],	false);
	

	if(ILPReadFromFile(key, iniPath) == -1){
		// iniファイルへ書き込み
		ILPWriteToFile(key, iniPath, "\r\n");//読み込めなかったら自動で作成
	}
	
	switch(sceKernelInitKeyConfig()){
		case PSP_INIT_KEYCONFIG_VSH:
			config.swap = swap[0];
			break;
		case PSP_INIT_KEYCONFIG_GAME:
			config.swap = swap[1];
			break;
		case PSP_INIT_KEYCONFIG_POPS:
			config.swap = swap[2];
			break;
		default:
			config.swap = swap[1];
			break;
	}
	return;
}

/*----------------------------------------------------------------------------*/

void ClearCaches(void)
{
	sceKernelDcacheWritebackAll();
	sceKernelIcacheClearAll();
}

int Patch_sceImposeGetParam()
{
	u32 text_addr, text_end;
	SceModule2 *mod = sceKernelFindModuleByName("sceImpose_Driver");
	if(mod == NULL)return -1;
	
	text_addr = mod->text_addr;
	text_end = mod->text_addr + mod->text_size;
	
	for(; text_addr < text_end; text_addr += 4)
	{
		if(_lw(text_addr + 12) == 0x8DC40160)
		{
			break;
		}
	}
	if(text_addr < text_end)
	{
		_sw(0x3C0E0000 | (((int)&slide_action >> 16) & 0xFFFF), text_addr);//lui $t6, 0xXXXX
		_sw(0x35CE0000 | ((int)&slide_action & 0xFFFF), text_addr+4);//ori $t6, $t6, 0xXXXX
		_sw(0x8DC40000, text_addr+12);//lw $a0, 0($t6)
		
		ClearCaches();
		return 0;
	}
	return -1;
}


int main_thread( SceSize arglen, void *argp ){
	SceCtrlData padData;
	
	while( sceKernelFindModuleByName("sceKernelLibrary") == NULL ){
		sceKernelDelayThread(1000);
	}
	
	if(sceKernelGetModel() != 4 || sceImposeGetParam(PSP_IMPOSE_SLIDE_ACTION)){
		sceKernelSelfStopUnloadModule(0, 0, NULL);
		return 0;
	}
	
	roadIni(argp);
	Patch_sceImposeGetParam();
	
	sceCtrlPeekBufferPositive(&padData, 1);
	
	while(1){
	
		while((padData.Buttons & PSP_CTRL_SLIDE) == 0){
			sceKernelDelayThread(200*1000);
			sceCtrlPeekBufferPositive(&padData, 1);
		}
		
		slide_action = 1;
		
		while((padData.Buttons & PSP_CTRL_SLIDE) != 0){
			sceKernelDelayThread(200*1000);
			sceCtrlPeekBufferPositive(&padData, 1);
		}
		
		if(padData.Buttons & config.shutdown_button){
			scePowerRequestStandby();
		} else if(((padData.Buttons & config.swap_button) != 0) ^ config.swap){
			scePowerRequestSuspend();
		} else {
			slide_action = 0;
		}
	}
	return 0;
}

/*----------------------------------------------------------------------------*/

int module_start( SceSize arglen, void *argp )
{
	SceUID thid = sceKernelCreateThread( "main_thread", main_thread, 30, 0x1000, 0, 0 );
	if( thid ) sceKernelStartThread( thid, arglen, argp );

  return 0;
}

int module_stop( void )
{
	return 0;
}


